@@ -78,9 +78,9 @@ detector_scope = freephil.parse(
       image_size = 4148 4362
         .type = ints(size = 2)
         .help = "Image size in pixels: (fast, slow)"
-      exposure_time = 0.004s
+      exposure_time = None
         .type = str
-        .help = "Nominal exposure time, if unit is not specified defaults to seconds"
+        .help = "Nominal exposure time, eg 0.004s, if unit is not specified defaults to seconds"
       axes = two_theta det_z
         .type = strings
         .help = "Axis names for the detector axes. The detector sits on the last one."
